﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;

namespace KSP_Part_Lister
{
    public partial class FormMain : Form
    {
        Settings settings = new Settings("settings.txt");
        string kspDirectory = "";
        List<string> fields = new List<string>();
        int sortColumn = -1;
        bool sortAscending = true;

        public FormMain()
        {
            InitializeComponent();

            SettingsItem item = settings.GetItem("window");
            if (item != null)
            {
                int width = Convert.ToInt32(item.GetProperty("width"));
                if (width > 0)
                    this.Width = width;

                int height = Convert.ToInt32(item.GetProperty("height"));
                if (height > 0)
                    this.Height = height;
            }

            progressBar.Location = Point.Empty;
            progressBar.Size = this.DisplayRectangle.Size;
            progressBar.BringToFront();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            kspDirectory = Directory.GetParent(Application.ExecutablePath).FullName;
            if (!kspDirectory.EndsWith("\\"))
                kspDirectory += "\\";

            if (Directory.Exists(kspDirectory + "Parts"))
            {
                kspDirectory = Directory.GetParent(Application.ExecutablePath).FullName;
                if (!kspDirectory.EndsWith("\\"))
                    kspDirectory += "\\";
            }
            else
            {
                try
                {
                    kspDirectory = settings.GetItem("kspdir").GetProperty("directory");
                    if (!kspDirectory.EndsWith("\\"))
                        kspDirectory += "\\";
                }
                catch { }
            }

            string[] partsDir = null;
            try
            {
                partsDir = Directory.GetDirectories(kspDirectory + "Parts");
            }
            catch
            {
                DialogResult result = System.Windows.Forms.DialogResult.OK;
                while (result == System.Windows.Forms.DialogResult.OK)
                {
                    FolderBrowserDialog folder = new FolderBrowserDialog();
                    folder.Description = "KSP Root Directory";
                    result = folder.ShowDialog(this);

                    if (result != System.Windows.Forms.DialogResult.OK)
                        break;

                    kspDirectory = folder.SelectedPath;
                    if (!kspDirectory.EndsWith("\\"))
                        kspDirectory += "\\";
                    try
                    {
                        partsDir = Directory.GetDirectories(kspDirectory + "Parts");
                        break;
                    }
                    catch { }
                }
                if (result != System.Windows.Forms.DialogResult.OK)
                    Application.Exit();

                SettingsItem kspDir = settings.GetItem("kspdir");
                if (kspDir == null)
                {
                    kspDir = new SettingsItem("setting");
                    kspDir.SetProperty("name", "kspdir");
                    kspDir.SetProperty("directory", kspDirectory);
                    settings.AddItem(kspDir);
                }
                settings.Save();
            }

            progressBar.Maximum = partsDir.Length;

            new Thread(() =>
                {
                    foreach (SettingsItem item in settings.Items)
                    {
                        if (item.Type == "field")
                        {
                            Invoke((MethodInvoker)delegate()
                            {
                                listParts.Columns.Add(item.GetProperty("heading"), Convert.ToInt32(item.GetProperty("width")));
                            });
                        }
                    }


                    foreach (string partDir in partsDir)
                    {
                        Invoke((MethodInvoker)delegate()
                        {
                            try
                            {
                                progressBar.PerformStep();
                            }
                            catch { }
                        });

                        ListViewItem part = new ListViewItem();
                        bool firstCol = true;

                        foreach (SettingsItem item in settings.Items)
                        {
                            if (item.Type == "field")
                            {
                                string partFile = partDir + "\\part.cfg";
                                string property = item.GetProperty("property");
                                string value = "";

                                if (property == "%partdir%")
                                {
                                    value = partDir.Split('\\')[partDir.Split('\\').Count() - 1];
                                }
                                else
                                {
                                    value = GetInfo(partFile, property);
                                    if (value == null & item.GetProperty("property2") != "")
                                        value = GetInfo(partFile, item.GetProperty("property2"));
                                }


                                try
                                {
                                    Invoke((MethodInvoker)delegate()
                                    {
                                        if (firstCol)
                                        {
                                            part.Text = value;
                                            firstCol = false;
                                        }
                                        else
                                        {
                                            part.SubItems.Add(value);
                                        }
                                    });
                                }
                                catch { break; }
                            }
                        }
                        try
                        {
                            Invoke((MethodInvoker)delegate()
                            {
                                listParts.Items.Add(part);
                            });
                        }
                        catch { break; }
                    }
                    try
                    {
                        Invoke((MethodInvoker)delegate()
                        {
                            progressBar.Hide();
                        });
                    }
                    catch { Application.ExitThread(); }
                }).Start();
        }

        private string GetInfo(string file, string key)
        {
            string retInfo = null;
            StreamReader sr = new StreamReader(file);
            while (true)
            {
                try
                {
                    string[] line = sr.ReadLine().Split('=');
                    if (line[0].Trim() == key)
                    {
                        retInfo = line[1].Trim();
                        break;
                    }
                }
                catch { break; }
            }
            sr.Close();
            return retInfo;
        }

        private void btnExportCSV_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.Filter = "Tab Seperated (*.csv)|*.csv|Tab Seperated (*.txt)|*.txt|Comma Seperated (*.csv)|*.csv|Comma Seperated (*.txt)|*.txt|All Files (*.*)|*.*";
            dialog.AddExtension = true;
            dialog.ShowDialog(this);
            string file = dialog.FileName;
            try
            {
                StreamWriter sw = new StreamWriter(file);
                string seperator = "\t";
                if (dialog.FilterIndex == 3 || dialog.FilterIndex == 4)
                    seperator = ",";

                string line = "";
                for (int i = 0; i < listParts.Columns.Count; i++)
                {
                    line += listParts.Columns[i].Text;
                    if (i < listParts.Columns.Count - 1)
                        line += seperator;
                }

                sw.WriteLine(line);

                foreach (ListViewItem item in listParts.Items)
                {
                    line = "";
                    for (int i = 0; i < item.SubItems.Count; i++)
                    {
                        line += item.SubItems[i].Text;
                        if (i < item.SubItems.Count - 1)
                            line += seperator;
                    }

                    sw.WriteLine(line);
                }
                sw.Close();
            }
            catch { }
        }

        void listParts_ColumnClick(object sender, System.Windows.Forms.ColumnClickEventArgs e)
        {
            if (sortColumn == e.Column)
                sortAscending = !sortAscending;

            listParts.ListViewItemSorter = new ListViewItemComparer(e.Column, sortAscending);

            listParts.Sort();

            sortColumn = e.Column;
        }
    }
}
