﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace KSP_Part_Lister
{
    class Settings
    {
        public bool IsLoaded
        {
            get;
            private set;
        }
        public string FileName
        {
            get;
            private set;
        }

        public List<SettingsItem> Items
        {
            get;
            private set;
        }

        public Settings(string fileName)
        {
            FileName = fileName;
            IsLoaded = Load();
        }

        public SettingsItem GetItem(string type, string propertyName, string propertyValue)
        {
            foreach (SettingsItem item in Items)
            {
                if (item.Type == type & item.GetProperty(propertyName) == propertyValue)
                    return item;
            }
            return null;
        }

        public SettingsItem GetItem(string type, string name)
        {
            foreach (SettingsItem item in Items)
            {
                if (item.Type == type & item.Name == name)
                    return item;
            }
            return null;
        }

        public SettingsItem GetItem(string name)
        {
            foreach (SettingsItem item in Items)
            {
                if (item.Name == name)
                    return item;
            }
            return null;
        }

        public void AddItem(SettingsItem item)
        {
            Items.Add(item);
        }

        public void Save()
        {
            StreamWriter stream = new StreamWriter(FileName);

            int i = 0;
            foreach (SettingsItem item in Items)
            {
                item.SaveToStream(stream);
                i++;

                if (i < Items.Count())
                    stream.WriteLine("\r\n");
            }

            stream.Close();
        }

        public bool Load()
        {
            Items = new List<SettingsItem>();

            if (!File.Exists(FileName))
                return false;

            StreamReader stream = new StreamReader(FileName);

            while (!stream.EndOfStream)
            {
                SettingsItem item = GetNextItem(stream);
                if (item != null)
                    Items.Add(item);
            }

            stream.Close();

            return true;
        }

        private SettingsItem GetNextItem(StreamReader stream)
        {
            string line = "";

            while (!stream.EndOfStream)
            {
                line = RemoveComments(stream.ReadLine());

                if (line != "")
                    break;
            }

            if (stream.EndOfStream)
                return null;

            SettingsItem item = new SettingsItem(line);

            while (RemoveComments(stream.ReadLine()) != "{" & !stream.EndOfStream) { }

            while (line != "}" & !stream.EndOfStream)
            {
                line = RemoveComments(stream.ReadLine());
                if (line != "" & line != "}")
                    item.SetPropertyLine(line);
            }

            return item;
        }

        private string RemoveComments(string line)
        {
            return line.Split("//".ToCharArray())[0].Trim();
        }
    }
}
