﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace KSP_Part_Lister
{
    public class SettingsItem
    {
        public string Type
        {
            get;
            private set;
        }

        public string Name
        {
            get { return GetProperty("name"); }
        }

        public Dictionary<string, string> Properties
        {
            get;
            private set;
        }

        public SettingsItem(string type)
        {
            Properties = new Dictionary<string, string>();
            Type = type;
        }

        public string GetProperty(string name)
        {
            string value;

            if (Properties.TryGetValue(name, out value))
                return value;

            return null;
        }

        public string GetPropertyLine(string name)
        {
            string value;

            if (Properties.TryGetValue(name, out value))
                return name + " = " + value;

            return null;
        }

        public void SetProperty(string name, string value)
        {
            Properties.Remove(name);
            Properties.Add(name, value);
        }

        public void SetPropertyLine(string line)
        {
            Properties.Add(line.Split('=')[0].Trim(), line.Split('=')[1].Trim());
        }

        public bool RemoveProperty(string name)
        {
            return Properties.Remove(name);
        }

        public void SaveToStream(StreamWriter stream)
        {
            stream.WriteLine(Type);
            stream.WriteLine("{");

            foreach (string name in Properties.Keys)
            {
                string value;
                Properties.TryGetValue(name, out value);
                stream.WriteLine("\t" + name + " = " + value);
            }
            stream.Write("}");
        }
    }
}
